# -*- coding: cp1252 -*-
'''
##
## Script:
##
## Purpose: function library
##          convert hoogtelijn featureclasses to routes and events conform Baseline 5 dataprotocol
##
## usage: Call from another python script:
##
## Author: Koen Volleberg (Lievense)
## Author: Jan den Daas (WSP)
## 
## 
## version: check script bim6_versipninfo.py
##

'''
import os
import arcpy
import bim6_versioninfo
import bim6_2d_to_3d
import bim6_lines2routes
from generalFunctions import *
scriptversion = bim6_versioninfo.version
def booleanizeInput(inputParameter):
    if inputParameter == True or inputParameter == False:
        return inputParameter
    elif str(inputParameter) == '1' or str(inputParameter.lower()) == 'true':
        return True
    else:
        return False   

    
def AddError(er):
    arcpy.AddError(er)
    print(er)

    
def AddMessage(mes):
    arcpy.AddMessage(mes)
    print(mes)
    
    
def linesandpoints2protocol(inlines, tolerance, swan=False):
    # bepalen locatie workspace en naam type lijnen
    desc = arcpy.Describe(inlines)
    locatie = desc.path
    fgdb = os.path.dirname(locatie)
    
    lineName = desc.basename.lower()
    linePath = os.path.join(locatie, lineName).lower()

    templatePath = GetBaselineMeasureTemplateLocation()# 'd:/temp/baseline.gdb'
    if not templatePath:
        AddError('Baseline template not found in user directory. This means Baseline 6 has never been run on this machine by this user. Please start ArcMap and enable Baseline 6 once before using this tool.')
        return
     
    elevFD = os.path.join(templatePath, r'baseline.gdb/elevation')
    if arcpy.Exists(os.path.join(arcpy.Describe(inlines).path, 'elevation_model_terrain')) and not swan:
        AddError('Selected feature class is part of a variant. This tool is meant for Baseline measures only.')
        return
        
    lineDict = {'terrain_edge_lines': ['terrain_edge_3d_lines',None, 'ELEVATION_CREST', ['CHARACTERISTICS']],
                'terrain_jump_lines': ['terrain_jump_3d_routes', 'terrain_jump_3d_events', 'ELEVATION_CREST', ['CHARACTERISTICS']],
                'elevated_line_lines': ['elevated_line_routes', 'elevated_line_events', None, None],
                'bridge_lines': ['bridge_routes', 'bridge_events', None, None]
                }
                
    if not lineName in lineDict:
        AddError("Selected lines not recognized as temporary Baseline '<baseline_name>_lines' feature class. Operation aborted.")
        return
    else:
        success = False
        pointPath = linePath[:-6] + "_points"
        
        outputNames = lineDict[lineName]
        t = os.path.join(elevFD, outputNames[0])
        outlines = os.path.join(locatie, outputNames[0])
        outEventName = outputNames[1]
        extraFields = outputNames[3] 
        if outEventName:
            # has events            
            outevents = os.path.join(fgdb, outEventName)
            if outputNames[2]:
                # Is 3D route
                outTempLines = outlines.replace('3d_routes', '3d_lines')
                if arcpy.Exists(outTempLines):
                    arcpy.Delete_management(outTempLines)                           
                bim6_2d_to_3d.create3dlinefc(inlines, pointPath, outputNames[2], outTempLines, t, extraFields)
                if outEventName == 'terrain_jump_3d_events':
                    bim6_lines2routes.lines2routes(fgdb, pointPath, outTempLines, outevents, tolerance,type=False)   
                else:
                    bim6_lines2routes.lines2routes(fgdb, pointPath, outTempLines, outevents, tolerance)
                
                # Delete the 3D Z field from events
                arcpy.DeleteField_management(outevents, outputNames[2])
                success=True
            else:
                # Is 2D route
                if outEventName == 'bridge_events':
                    bim6_lines2routes.lines2routes(fgdb, pointPath, linePath, outevents, tolerance, bridge=True)
                else:
                    bim6_lines2routes.lines2routes(fgdb, pointPath, linePath, outevents, tolerance)   
                success = True                
        else:
            # alleen conversie naar 3d lines
            if arcpy.Exists(outlines):
                arcpy.Delete_management(outlines)            
            success = bim6_2d_to_3d.create3dlinefc(inlines, pointPath, outputNames[2], outlines, t, extraFields)                
        if success:        
            AddMessage('Conversion succeeded. Original data will be deleted.')
            arcpy.Delete_management(inlines)
            arcpy.Delete_management(pointPath)
                
            
if __name__ == '__main__':
    inlines = arcpy.GetParameterAsText(0)
    toler = arcpy.GetParameter(1)
    try:
        swan = booleanizeInput(arcpy.GetParameter(2))
    except:
        swan = False
    try:
        batch = booleanizeInput(arcpy.GetParameter(3))
    except:
        batch = False    
    
        
    if batch:
        fgdbs = os.path.join(inlines, 'baseline.gdb')
        for batchlines in ['elevation/terrain_edge_lines','elevation/terrain_jump_lines','elevation/elevated_line_lines','locations/bridge_lines']: 
            if arcpy.Exists(os.path.join(fgdbs,batchlines)):
                linesandpoints2protocol(os.path.join(fgdbs,batchlines), toler,swan=False)
    else:
        linesandpoints2protocol(inlines, toler,swan)
