import os, datetime, string
import arcpy

#Overschrijven aan
arcpy.env.overwriteOutput = 1

def AddMessage(mes):
    print(mes)
    arcpy.AddMessage(mes)

#   add 'to' and 'from' fields to table and fill them
def addfields (table, fr_fld, to_fld):
    arcpy.AddField_management(table, fr_fld, "LONG")
    arcpy.AddField_management(table, to_fld, "LONG")
    fields = [fr_fld, to_fld, "NUMBER"]
    rows = arcpy.da.UpdateCursor(table, fields,sql_clause=(None, "ORDER BY NUMBER ASC"))

    nummer = 0
    count = 1
    for row in rows:
        nummerPrev = nummer
        nummer = row[2]
        if nummer != nummerPrev:
            count = 1
        else:
            count = count + 1
        row[0] = (count*100)-99
        row[1] = (count*100)
        rows.updateRow(row)
    del rows
    return

#   add field "kenmerk" and fill
def addKenmerk(out_routes, in_lines):
    arcpy.AddField_management(out_routes, "CHARACTERISTICS", "TEXT", "" , "", "100")

    tmpView = "tmpView"

    aline = out_routes.replace("\\","/")
    aline = aline.replace("//","/")
    alist = aline.split("/")
    if (len(alist) > 0):
        out_routes_fc = alist[len(alist)-1]

    aline = in_lines.replace("\\","/")
    aline = aline.replace("//","/")
    alist = aline.split("/")
    if (len(alist) > 0):
        in_lines_fc = alist[len(alist)-1]

    arcpy.MakeTableView_management(out_routes, tmpView, "", "", "NUMBER NUMBER VISIBLE,CHARACTERISTICS CHARACTERISTICS VISIBLE")
    arcpy.AddJoin_management(tmpView, "NUMBER", in_lines, "NUMBER", "KEEP_ALL")
    arcpy.CalculateField_management(tmpView, ("%s.CHARACTERISTICS" % out_routes_fc), ("!%s.CHARACTERISTICS!"% in_lines_fc), "PYTHON")

    arcpy.Delete_management(tmpView)

    return
      
def addName(out_routes, in_lines):
    arcpy.AddField_management(out_routes, "NAME", "TEXT", "" , "", "100")

    tmpView = "tmpView"

    aline = out_routes.replace("\\","/")
    aline = aline.replace("//","/")
    alist = aline.split("/")
    if (len(alist) > 0):
        out_routes_fc = alist[len(alist)-1]

    aline = in_lines.replace("\\","/")
    aline = aline.replace("//","/")
    alist = aline.split("/")
    if (len(alist) > 0):
        in_lines_fc = alist[len(alist)-1]

    arcpy.MakeTableView_management(out_routes, tmpView, "", "", "NUMBER NUMBER VISIBLE,NAME NAME VISIBLE")
    arcpy.AddJoin_management(tmpView, "NUMBER", in_lines, "NUMBER", "KEEP_ALL")
    arcpy.CalculateField_management(tmpView, ("%s.NAME" % out_routes_fc), ("!%s.NAME!"% in_lines_fc), "PYTHON")

    arcpy.Delete_management(tmpView)

    return

#   add field "type" and fill
def addType(out_routes, in_lines):
    
    arcpy.AddField_management(out_routes, "TYPE", "LONG")
    
    # Set domain
    domainName = False
    if(out_routes.endswith('elevated_line_routes_temp')):
        domainName = 'elevated_line_type'
    elif(out_routes.endswith('terrain_jump_3d_routes_temp')):
        domainName = 'terrain_jump_type'
    if domainName:
        arcpy.AssignDomainToField_management(out_routes, "TYPE", domainName)
    arcpy.AddMessage("assign domainname '{}' to FC '{}'".format(domainName, out_routes))

    tmpView = "tmpView2"

    aline = out_routes.replace("\\","/")
    aline = aline.replace("//","/")
    alist = aline.split("/")
    if (len(alist) > 0):
        out_routes_fc = alist[len(alist)-1]

    aline = in_lines.replace("\\","/")
    aline = aline.replace("//","/")
    alist = aline.split("/")
    if (len(alist) > 0):
        in_lines_fc = alist[len(alist)-1]

    arcpy.MakeTableView_management(out_routes, tmpView, "", "", "NUMBER NUMBER VISIBLE,TYPE TYPE VISIBLE")
    arcpy.AddJoin_management(tmpView, "NUMBER", in_lines, "NUMBER", "KEEP_ALL")
    arcpy.CalculateField_management(tmpView, ("%s.TYPE" % out_routes_fc), ("!%s.TYPE!"% in_lines_fc), "PYTHON")

    arcpy.Delete_management(tmpView)

    return


#   Delete duplicate events or events that are on the wrong route
#   Points that can not be converted to events are stored in a separate featureclass
#   tbl     = the new eventtable
#   idlist  = OBJECTID's of the original point featureclass
#   in_points = the point featureclass containing the original points
def deleteRedundantEvents (tbl, idList, in_points):
    rows = arcpy.da.UpdateCursor(tbl, field_names=["ID", "DISTANCE", "RID" ,"NUMBER"],
                                 sql_clause=(None, "ORDER BY ID, DISTANCE ASC"))
    
    for row in rows:
        point_id = row[0]
        rid = row[2]
        nummer = row[3]
        if (rid != nummer):
            # the point is located on the wrong route
            rows.deleteRow()
        else:
            if (idList.count(point_id) > 0):
                # OK, the id in the eventtable matches an id in the list. Delete this list entry and keep the event.
                idList.remove(point_id)
            else:
                # id does not exist (anymore) in the idlist: it must be a duplicated event. Row can be deleted.
                rows.deleteRow()
    del rows
    
    if (len(idList) > 0):
        AddMessage("The following " + str(len(idList)) + " points (objectid's) were not converted to events, these points will be saved to featureclass " + os.path.split(in_points)[1] + "_locate_errors: ")
        idsToDelete = ""
        for i in idList:
            AddMessage(i)
            if (len(idsToDelete) == 0):
                idsToDelete = str(i)
            else:
                idsToDelete = idsToDelete + "," + str(i)
        # copy the non-matching points from the original featureclass to another featureclass with the same name suffixed with "_locate_errors"
        arcpy.FeatureClassToFeatureClass_conversion(in_points, os.path.split(in_points)[0], os.path.split(in_points)[1] + "_locate_errors", "id in (" + idsToDelete + ")")
        return False
    else:
        return True

# add field with copy of objectid and put all the id's in a list
def addOIDtoPointsFC(pointsfc):
    arcpy.AddField_management(pointsfc, "ID", "LONG")
    arcpy.CalculateField_management(pointsfc, "ID", "!OBJECTID!", "PYTHON_9.3")
    idList = []
    rows = arcpy.da.SearchCursor(pointsfc, "OID@")
    for row in rows:
        idList.append(row[0])
    del rows
    return idList


def lines2routes(workspace, in_points, in_lines, out_events, tolerance,bridge=False, type=True):
    arcpy.env.workspace = workspace   
#   names for events and routes
    out_routes = ""

    out_routes = in_lines.replace("_lines", "_routes_temp")
    desc = arcpy.Describe(in_points)
    ptName = desc.name

    AddMessage("Working on featuredataset %s\n" %(os.path.split(in_points)[0]))
    AddMessage("Preprocessing " + out_events.replace("_events","") + "...")
    
#   add fields to line featureclasses
    fr_fld = "MFROM"
    to_fld = "MTO"
    addfields(in_lines,fr_fld,to_fld)

#   Create routes from lines. The TWO_FIELD option will be used to set the measures.
    rid = "number"
    AddMessage("Creating routes...")
    arcpy.CreateRoutes_lr(in_lines, rid, out_routes, "TWO_FIELDS", fr_fld, to_fld)
    if bridge:
        AddMessage("Transfer NAME to routes...")
        addName(out_routes, in_lines)
    AddMessage("Transfer CHARACTERISTICS to routes...")
    addKenmerk(out_routes, in_lines)
    if type and not bridge:
        AddMessage("Transfer TYPE to routes...")
        addType(out_routes, in_lines)

     
#   Make point event tables.
    rid = "NUMBER"
    props = "rid POINT MEASURE "
    idList = addOIDtoPointsFC(in_points)
    AddMessage("Creating events...")
    arcpy.LocateFeaturesAlongRoutes_lr(in_points, out_routes, rid, tolerance, out_events, props, "ALL", "DISTANCE" )

#   delete redundant rows
    AddMessage("Check events and delete wrongly placed and redundant events...")
    checkok = False
    if (deleteRedundantEvents(os.path.join(workspace,out_events), idList, in_points)):
        AddMessage("Check events = OK")
        checkok = True
    arcpy.DeleteField_management(out_events,"id; rid; distance; old_fid")

#   multipart to singlepart
    AddMessage("Creating singleparts from the routes...")
    out_routes_single = out_routes.replace("_routes_temp", "_routes")
    
    arcpy.MultipartToSinglepart_management(out_routes, out_routes_single)
    arcpy.Delete_management(out_routes)

# Delete unwanted fields    
    AddMessage("Delete unwanted fields...")
    desc = arcpy.Describe(out_routes_single)
    oidFieldName = string.lower(desc.OidFieldName)
    shpFieldName = string.lower(desc.ShapeFieldName)
    if bridge:
        allowedFields = ["number","characteristics","name",oidFieldName,shpFieldName, "shape_length","shape_area"] 
    else:
        allowedFields = ["number","characteristics","type",oidFieldName,shpFieldName, "shape_length","shape_area"]
    listFields = arcpy.ListFields(out_routes_single)
    delFields = []
    for fld in listFields:
        if not fld.name.lower() in allowedFields:
            delFields.append(fld.name)
    # for delField in delFields:
    arcpy.DeleteField_management(out_routes_single,delFields)
    del listFields, delFields
            
    if arcpy.Exists(out_routes):
        arcpy.AddError("Conversion not successful. Temp file still exists. Check logs and data for issues. Locking of feature classes might cause problems.\nOriginal data not deleted.")
    elif not checkok:
        arcpy.AddError("Conversion not successful. Invalid events found. Check logs and data for issues.")
    else:
        arcpy.Delete_management(in_points)
        arcpy.Delete_management(in_lines)
        
        AddMessage("Done.")
    return True
