# -*- coding: cp1252 -*-
'''
##
## Script:
##
## Purpose: Renumbering of routes and coressponding events
##
## usage: Call from another python script:
##
## Author: Jan den Daas (WSP)
## 
## 
## version: check script bim6_versipninfo.py
##

'''
import os
import arcpy
import bim6_versioninfo
import bim6_2d_to_3d
import bim6_lines2routes
import numpy as np
scriptversion = bim6_versioninfo.version
def booleanizeInput(inputParameter):
    if inputParameter == True or inputParameter == False:
        return inputParameter
    elif str(inputParameter) == '1' or str(inputParameter.lower()) == 'true':
        return True
    else:
        return False   

    
def AddError(er):
    arcpy.AddError(er)
    print(er)

    
def AddMessage(mes):
    arcpy.AddMessage(mes)
    print(mes)

class FeatureClassToConvert():
    events = None
    eventFields = None
    isValid = True
    
    dictLineTypes = {
        'terrain_edge_3d_lines': '3dline',
        'terrain_jump_3d_routes': '3droute',
        'elevated_line_routes': 'route',
        'bridge_routes': 'route',
    }
    
    dictFields = {
        'elevated_line_events': ['Shape@XY', 'NUMBER', 'MEASURE', 'ELEVATION_CREST', 'ELEVATION_LEFT', 'ELEVATION_RIGHT', 'SLOPE_LEFT', 
                         'SLOPE_RIGHT','CREST_WIDTH', 'ROUGHNESS_CODE'],
        'terrain_jump_3d_events': ['Shape@XY',  'NUMBER', 'MEASURE', 'ELEVATION_LEFT', 'ELEVATION_RIGHT', 'SLOPE_LEFT', 
                         'SLOPE_RIGHT','CREST_WIDTH', 'ROUGHNESS_CODE'],
        'terrain_edge_3d_lines': ['Shape@', 'NUMBER', 'CHARACTERISTICS'],
        'terrain_jump_3d_routes': ['Shape@', 'NUMBER', 'CHARACTERISTICS'],
        'elevated_line_routes': ['Shape@', 'NUMBER', 'TYPE', 'CHARACTERISTICS'],
        'bridge_routes': ['Shape@', 'NUMBER', 'NAME', 'CHARACTERISTICS'],
        'bridge_events': ['Shape@XY', 'NUMBER', 'MEASURE','NAME', 'CHARACTERISTICS', 'DIAMETER', 'CP']
                         }
                     
    def __init__(self, inFeatures):
        desc = arcpy.Describe(inFeatures)
        self.name = desc.basename.lower()
        
        self.type = self.dictLineTypes.get(self.name, None)
        self.fields = self.dictFields.get(self.name, None)
        self.location = desc.path
        self.fullPath = os.path.join(self.location, self.name)
        self.fgdb = os.path.dirname(self.location)        
        if self.type == '3dline':
            self.hasEvents = False
        elif self.type == None:
            self.hasEvents = False
            self.isValid = False
            return
        else:
            self.hasEvents = True
            eventsName = self.name.replace("routes","events")    
            self.events = os.path.join(self.fgdb, eventsName)                 
            self.eventFields = self.dictFields[eventsName]
                
        if self.type == '3dline':
            subName = self.name[:-9]
        elif self.type == '3droute':
            subName = self.name[:-10]
        else: 
            subName = self.name[:-7]
            
        #route en punten wegschrijven in gdb
        self.outputPointName = subName + "_points"
        self.outputLineName = subName + "_lines"
        self.outputTempName = subName + "_temp"
        self.pointFc = os.path.join(self.location, self.outputPointName)
        self.lineFc = os.path.join(self.location, self.outputLineName)
        self.tempFc = os.path.join(self.location, self.outputTempName)               

def numberRoute(feat2convert):


        
    AddMessage("Start conversion of normal(2D) routes and events")   
    
    # Convert routes to lines (sipmle copy features)
    AddMessage("Converting routes...")
    
    uc = arcpy.da.UpdateCursor(feat2convert.fullPath, field_names=["NUMBER"],sql_clause=(None, "ORDER BY NUMBER"))
    
    hasEvents=True
    if not feat2convert.hasEvents:
        AddMessage('Selected feature class does not have corresponding events.')
        hasEvents=False
        
    index = 1
    reference=[]
    for route in uc:
        if hasEvents:
            reference.append([route[0],index])
        route[0]=index
        uc.updateRow(route) 
        index+=1
        
    if hasEvents and len(reference)>0:
        reference = np.asarray(reference)     
        numbers = reference[:,0]
        # Conversion of events
        AddMessage("Converting events...")
        uc2 = arcpy.da.UpdateCursor(feat2convert.events, field_names=["NUMBER"])
        for event in uc2:
            index=np.searchsorted(numbers, event[0])
            event[0]=reference[index,1]
            uc2.updateRow(event) 


def CreateNumbers(inFc):
    AddMessage("Script: %s" % os.path.basename(sys.argv[0]))
    AddMessage("Version: %s" % scriptversion)
    AddMessage("WSP / Deltares, 2022\n")
    
    # bepalen locatie workspace en naam type lijnen
    fc2Convert = FeatureClassToConvert(inFc)
    if arcpy.Exists(os.path.join(fc2Convert.location, 'elevation_model_terrain')):
        AddError('Selected feature class is part of a variant. This tool is meant for Baseline measures only.')
        return None
     
    lineType = fc2Convert.type
    
    if lineType == 'route':
        numberRoute(fc2Convert)          
    elif lineType == '3droute':
        numberRoute(fc2Convert)  
    elif lineType == '3dline':
        numberRoute(fc2Convert)
    else:
        AddError('Selected feature class is not a valid Baseline 6 feature class')
        return None
        


if __name__ == "__main__":
    inFc = arcpy.GetParameterAsText(0)
    batch = arcpy.GetParameter(1)
        
    if batch:
        fgdbs = os.path.join(inFc, 'baseline.gdb')
        for batchroutes in [r'elevation/terrain_edge_3d_lines',r'elevation/terrain_jump_3d_routes',r'elevation/elevated_line_routes',r'locations/bridge_routes']:
            if arcpy.Exists(os.path.join(fgdbs,batchroutes)):
                CreateNumbers(os.path.join(fgdbs,batchroutes))
    else:
        CreateNumbers(inFc)
