import os
import arcpy
import settings
from generalFunctions import *
from os.path import join

"""
    
Baseline 6 Input Module

AIM
    - Remove empty FC

Lievense 2017
Niels Mulder

Based on bim5_cleandatabase.py by Koen Volleberg, Lievense
Based on Baseline 5 tot 6 converter by Niels Mulder, Lievense

"""

arcpy.env.overwriteOutput = True


def cleanBaselineDir(baselineDir):

    AddMessage('Baseline Input Module Tool: {}. Version {}'.format(os.path.basename(sys.argv[0]), settings.getVersion()))
    if not settings.isDef():
        AddMessage('This is a beta version. Do not use in production.')
    AddMessage('Lievense {}'.format(settings.getVersionDate()))

    # Set workspace
    arcpy.env.workspace = os.path.join(baselineDir, 'baseline.gdb')
      
    #Loop over tables
    tables = arcpy.ListTables()
    for table in tables:
        path = os.path.join(baselineDir, 'baseline.gdb', table)
        dataInLength = countFeatures(path)
        if dataInLength == 0:
            AddMessage('{} is empty and will be removed.'.format(table))
            arcpy.Delete_management(path)
    
 
    # List datasets
    datasets = arcpy.ListDatasets()

    # Add None to datasets. Those points to the files not in a dataset
    datasets = [''] + datasets if datasets is not None else []

    # Loop over datasets
    for dataset in datasets:
        for featureClass in arcpy.ListFeatureClasses(feature_dataset=dataset):
            path = os.path.join(arcpy.env.workspace, dataset, featureClass)

            # Count
            dataInLength = countFeatures(path)

            # Delete if empty
            if dataInLength == 0:
                AddMessage('{}/{} is empty and will be removed.'.format(dataset, featureClass))
                arcpy.Delete_management(os.path.join(baselineDir, 'baseline.gdb', dataset, featureClass))
    


if __name__ =='__main__':
    baselineDir = arcpy.GetParameterAsText(0)
    cleanBaselineDir(baselineDir)
