import os
import arcpy
import settings
from generalFunctions import *
from os.path import join

"""
    
Baseline 6 Input Module

AIM
    - Create erase and append lists.

Lievense 2017
Niels Mulder

Based on bim5_createLijsten.py by Koen Volleberg, Lievense
Based on Baseline 5 tot 6 converter by Niels Mulder, Lievense

"""

arcpy.env.overwriteOutput = True


def CreateLists(baselineDir):

    AddMessage('Baseline Input Module Tool: {}. Version {}'.format(os.path.basename(sys.argv[0]), settings.getVersion()))
    if not settings.isDef():
        AddMessage('This is a beta version. Do not use in production.')
    AddMessage('Lievense {}'.format(settings.getVersionDate()))
    
    # Set workspace
    arcpy.env.workspace = os.path.join(baselineDir, 'baseline.gdb')

    # List datasets
    datasets = arcpy.ListDatasets()

    # Add None to datasets. Those points to the files not in a dataset
    datasets = [''] + datasets if datasets is not None else []

    # Space to store append and erase data
    listAppend = []
    listErase = []
    
    sea=False
    #Add elevation_raster
    if arcpy.Exists("elevation_raster"):
        listAppend.append("elevation_raster")
        sea=True
        
    # Loop over datasets
    AddMessage('Find all existing Feature Classes.')
    for dataset in datasets:
        for featureClass in arcpy.ListFeatureClasses(feature_dataset=dataset):
            path = os.path.join(arcpy.env.workspace, dataset, featureClass)
            if sea and dataset=="elevation":
                if "points" in featureClass or "edge" in featureClass:
                    continue
            # Count
            dataInLength = countFeatures(path)

            # Delete if empty
            if dataInLength > 0:
                if dataset == 'erase_polygons':
                    listErase.append('{}/{}'.format(dataset, featureClass.replace('_routes', '')))
                else:
                    listAppend.append('{}/{}'.format(dataset, featureClass.replace('_routes', '')))
    


    
    # Make dirs
    if not os.path.isdir(os.path.join(baselineDir, 'metainfo/lists')):
        os.makedirs(os.path.join(baselineDir, 'metainfo/lists'))
    
    # Write erase_list.txt
    AddMessage('Generate erase_list.txt')
    filehandle = open(os.path.join(baselineDir, 'metainfo/lists/erase_list.txt'), 'w')
    for line in listErase:
        filehandle.write(line+'\n')
    filehandle.close()

    # Write append_list.txt
    AddMessage('Generate append_list.txt')
    filehandle = open(os.path.join(baselineDir, 'metainfo/lists/append_list.txt'), 'w')
    for line in listAppend:
        filehandle.write(line+'\n')
    filehandle.close()


if __name__ =='__main__':
    baselineDir = arcpy.GetParameterAsText(0)
    CreateLists(baselineDir)
