import arcpy
import os
import datetime as dt
import settings

"""
Baseline 6 (all tools)

generalFunctions.py

AIM
    - some function we use over and over again

Lievense  2017
Niels Mulder

20200507 Jan den Daas
    Added sea template directories
20230213 Jan den Daas
    New way of finding appdata directory

"""

# See settings.py for instructions on logFileHandle

def AddMessage(mes, in_log=True):
    now = dt.datetime.now()
    strNow = now.strftime('%H:%M:%S')
    mes = '{time} - INFO: {mes}'.format(time=strNow, mes=mes)
    arcpy.AddMessage(mes)

    if in_log and settings.logFileHandle:
        settings.logFileHandle.write(mes + '\n')
    pass


def AddError(mes, in_log=True):
    now = dt.datetime.now()
    strNow = now.strftime('%H:%M:%S')
    mes = '{time} - ERROR: {mes}'.format(time=strNow, mes=mes)
    arcpy.AddError(mes)

    if in_log and settings.logFileHandle:
        settings.logFileHandle.write(mes + '\n')
    pass


def AddWarning(mes, in_log=True):
    now = dt.datetime.now()
    strNow = now.strftime('%H:%M:%S')
    mes = '{time} - WARNING: {mes}'.format(time=strNow, mes=mes)

    arcpy.AddWarning(mes)
    if in_log and settings.logFileHandle:
        settings.logFileHandle.write(mes + '\n')
    pass


def countFeatures(path):
    arcpy.MakeTableView_management(path, 'tblToCount')
    dataInLength = int(arcpy.GetCount_management('tblToCount').getOutput(0))
    arcpy.Delete_management('tblToCount')
    return dataInLength


def getLocation():
    # Find Baseline template folder
    appdata = os.path.join(os.path.dirname(os.getenv("APPDATA")),"local")
    return appdata


def GetBaselineMeasureTemplateLocation(sea=False):
    loc = getLocation()
    baselineTemplateDir = os.path.join(loc,"Baseline/template/measure")
    baselineTemplateDirsea = os.path.join(loc,"Baseline/template/measure_sea")
    if sea:
        return baselineTemplateDirsea
    
    else:
        return baselineTemplateDir

def GetBaselineVariantTemplateLocation(sea=False):
    loc = getLocation()
    baselineTemplateDir = os.path.join(loc,"Baseline/template/variant")
    baselineTemplateDirsea = os.path.join(loc,"Baseline/template/variant_sea")
    
    if sea:
        return baselineTemplateDirsea
    
    else:
        return baselineTemplateDir
        
def GetBaselineScriptTemplate():
    loc = getLocation()
    baselineTemplateDir = os.path.join(loc,r"Baseline/Template/scriptTemplate.gdb")
    return baselineTemplateDir

def getBaselineRoot(inFc):
    if 'baseline.gdb' in inFc.lower():
        baselineRoot = inFc.split('baseline.gdb')[0]
        if os.path.exists(baselineRoot):
            return baselineRoot
    return None

def getMetainfoValue(key):
    if key in settings.getMetainfoValueMemory:
        return settings.getMetainfoValueMemory[key]
    elif not arcpy.Exists(settings.metainfoLocation):
        return False
    else:
        with arcpy.da.SearchCursor(settings.metainfoLocation, ['key','value'], "key = '{}'".format(key)) as cursor:
            for row in cursor:
                settings.getMetainfoValueMemory[row[0]] = row[1]
                return row[1]
    return False
