import arcpy
import settings
import os, math, random, datetime
from generalFunctions import *

"""
    
Baseline 6 Input Module

AIM
    - verplaatsen van punten die in een gelijkmatrig grid liggen, op zo'n manier dat random verplaatsingen
        plaatsvinden en de punten niet meer exact even ver uit elkaar liggen
        (en een nearest neighbour analyse maar 1 waarde teruggeeft

Lievense 2017
Koen Volleberg, Niels Mulder

Based on bim5_Moveregulargrid.py by Koen Volleberg, Lievense

"""

arcpy.env.overwriteOutput = 1


def movePointsToIrregularGrid(inputFC):
    dt = datetime.datetime
    baselineRoot = getBaselineRoot(inputFC)

    print baselineRoot
    
    if baselineRoot:        
        metainfo = os.path.join(baselineRoot, 'metainfo/logs')
        print metainfo
        if os.path.exists(metainfo):
            path = os.path.join(metainfo, 'movePointsToIrregularGrid.log')
            print(path)
            settings.logFileHandle = open(path, 'a')

            
    AddMessage("=======================================================") 
    AddMessage('Baseline Input Module Tool: {}. Version {}'.format(os.path.basename(sys.argv[0]), settings.getVersion()))
    if not settings.isDef():
        AddMessage('This is a beta version. Do not use in production.')
    AddMessage('Lievense {}'.format(settings.getVersionDate()))
    AddMessage("Start: %s\n" % dt.now().ctime())
    
    AddMessage("Feature class to be processed: {0}".format(os.path.basename(inputFC)))
    
    if countFeatures(inputFC) == 0:
        AddMessage("Selected feature class is empty.")
        AddMessage("Stopped at: %s" % dt.now().ctime())
        AddMessage("movePointsToIrregularGrid stopped")
    else:
        inDesc = arcpy.Describe(inputFC)

        if inDesc.ShapeType.lower() != "point":
            AddError("No point feature class selected.")
            AddMessage("Stopped at: %s" % dt.now().ctime())
            AddMessage("movePointsToIrregularGrid stopped")
        else:
            #try:
                workspace = os.path.join(baselineRoot, 'baseline.gdb')

                edit = arcpy.da.Editor(workspace)
                edit.startEditing(False, True)
                edit.startOperation()
            
                shapeField = inDesc.ShapeFieldName
                recCount = countFeatures(inputFC)
                rows = arcpy.da.UpdateCursor(inputFC, ["Shape@"])
                i = 1
                for row in rows:
                    shape = row[0]
                    pnt = shape.getPart()
                    x = pnt.X
                    y = pnt.Y
                    
                    deltaX = random.uniform(0.0001, 0.001)
                    deltaXY = random.uniform(0.0001, 0.001)        
                    deltaY = math.sqrt(math.fabs(deltaXY**2-deltaX**2))
                    pnt.X = x + deltaX
                    pnt.Y = y + deltaY

                    row[0] = pnt
                    rows.updateRow(row)
                    if (i%100) == 0:
                        AddMessage("Processing " + str(i) + " of " + str(recCount) + " points.", in_log=False)
                    i += 1
                    
                edit.stopOperation()
                edit.stopEditing(True)

                AddMessage(str(recCount) + " points processed.\n")
                AddMessage("Finished at: %s" % dt.now().ctime())
                AddMessage("movePointsToIrregularGrid succeeded")
            #except:
            #    AddError(arcpy.GetMessages(2))
            #    AddError("Stopped at: %s" % dt.now().ctime())
            #    AddError("movePointsToIrregularGrid stopped")
    AddMessage("=======================================================")
    try:
        log.close()
    except:
        pass

if __name__ == "__main__":
    points = arcpy.GetParameterAsText(0)
    
    #points = 'C:/Users/muldn/Desktop/splp_maatr_blanco/baseline.gdb/elevation/surfacelevel_points'
    
    movePointsToIrregularGrid(points)
