# -*- coding: cp1252 -*-
'''
    Script: bim5_TeenhoogtePrikker.py
    Versie: zie script bim5_versioninfo.py
    Doel: prikken teenhoogtes uit TIN

    Auteur: Koen Volleberg, Lievense
    Voor: Stichting Deltares
    
    Updates:
    Versie: 1
    Datum: 13112014
    Door: Koen Volleberg
    Wat: eerste volledige release

    Versie: 1.1
    Datum: 17042015
    Door: Koen Volleberg
    Wat: taludbreedte, taludhelling, kruinbreedte werden niet
        default gevuld. Bleven leeg. Worden nu met default waarden gevuld

    Versie: 2.0
    Datum: 19052015
    Door: Koen Volleberg
    Wat: optimalisatie naar ArcGIS 10.1

    Versie: 2.4
    Datum: 25102016
    Door: Koen Volleberg
    Wat: typo loodlijnen-shapeveld insertcursor gefixed
    
    Versie: 2.5
    Datum: 22122016
    Door: Koen Volleberg
    Wat: performance verbeteringen die het afleiden van grote maatregelen
        belemmerde.
            
    Versie: 2.8
    Datum: 28062017
    Door: Koen Volleberg
    - onnodige checks op alleen teenhoogte aanpassen eruit gehaald (want dit is
        de aard van dit script nu: alleen teenhoogte
    - removed some obsolete code
    - check nearestnotsimilar niet meer alleen op X, maar op echte afstand (xy)
    - voor bandijken wordt nu bepaald welke zijde van de dijk buiten het model valt
        (eerst werd voor elk punt de minimale hoogte gepakt (=3.8e-38), deze kon 0 zijn voor
         gebieden binnendijks)
    - kdtree script bluntly taken from SciPy source (see kdtree.py for license info)
        This one is far more performance-optimized. Effect is larger with larger datasets
    - CheckIn 3D analyst na afronden uitgezet, want merkte dat de licentie ook teruggegeven 
        werd als deze in ArcCCatalog nog gewoon aan stond. Was irritant.
        
    
    Version: 2.9
    Datum 20171127
    Door: Niels Mulder
    Update to Baseline 6
    
    Version: 3.0
    Datum 20220323
    Door: Jan den Daas
    Default NoData is -9999
    
    Version: 4.0
    Datum 20230302
    Door: Jan den Daas
    Default Fix loodlijnen 
    
'''
import os, sys
import math, numpy
import arcpy

from kdtree import KDTree
import datetime
arcpy.env.overwriteOutput = 1

import bim6_versioninfo
scriptversion = bim6_versioninfo.version
scriptyear = bim6_versioninfo.year

#Global settings
#veldnamen
kField = "ELEVATION_CREST"
rField = "ELEVATION_RIGHT"
lField = "ELEVATION_LEFT"
#tField = "TEENHOOGTE"
unField = "UNIQUEID"
nmField ="NUMBER"
kmField =  "CHARACTERISTICS"
vtField = "VERTEX"
kbField = "CREST_WIDTH"
tlField = "SLOPE_LEFT"
trField = "SLOPE_RIGHT"
rhField = "ROUGHNESS_CODE"
locField = "LOC"

defaultTalud = 4
defaultRuwheid = 0

useStandardDistance = False
#maxAfstand = 10

overwritePoints = True

class PointData:
    def __init__(self, pnt, num, vert, k, km):
        self.Point = pnt
        self.Nummer = num
        self.Vertex = vert
        self.Kruin = k
        self.Kenmerk = km
      
def setTeenSideIsKruin(fc, updatefield, nummer):
    whereclause = '"{fieldname}" = {nummer}'.format(fieldname=nmField, nummer=str(nummer))
    uc = arcpy.da.UpdateCursor(fc, field_names=[updatefield, kField],
                               where_clause=whereclause)
    for row in uc:
        row[0] = row[1]
        uc.updateRow(row)
    del uc
    
def featCount(f):
    result = arcpy.GetCount_management(f)
    count = int(result.getOutput(0))
    return count

def findfieldindex(cur, fieldname):
    if fieldname in cur.fields:
        return cur.fields.index(fieldname)
    else:
        return None

def getIntersectResults(inPoint,vertex,nummer,inAngle,inDist,intersectPoints,left=False, isBandijk=False , sea=False):
    if useStandardDistance or isBandijk:
    
        if sea and left:
            inAngle=(inAngle+180)%360
        if sea:
            inAngle=(inAngle+90)%360
            x1 = loodrechtXY(inPoint.X,inPoint.Y,inAngle, inDist)[0]
            y1 = loodrechtXY(inPoint.X,inPoint.Y,inAngle, inDist)[1]
            coords = [x1,y1]
        elif left:
            x1 = inPoint.X - loodrechtX(inAngle, maxAfstand)
            y1 = inPoint.Y + loodrechtY(inAngle, maxAfstand)
            coords = [x1,y1]
        else:
            x1 = inPoint.X + loodrechtX(inAngle, maxAfstand)
            y1 = inPoint.Y - loodrechtY(inAngle, maxAfstand)
            coords = [x1,y1]
        return coords
    else:
            
        if left == True:
            strLoc = "l"
        else:
            strLoc = "r"
        strSelect = (""" "%s"=%s AND "VERTEX"=%s AND "LOC" = '%s' """ %(nmField, nummer, vertex, strLoc))

        inc = arcpy.da.SearchCursor(intersectPoints, field_names=["Shape@XY"], where_clause=strSelect)

        nears = []
        for inr in inc:
            xy = inr[0]
            nears.append([xy[0], xy[1]])
        del inc

        coords = []

        if len(nears) > 0:
            tree = KDTree(nears)
        
            nearestIndices = tree.query([inPoint.X, inPoint.Y], k=5, distance_upper_bound=maxAfstand)[1]
            #Hier evt bepalen dat als niks gevonden, dan 10m loodrecht op lijn
            if len(nearestIndices) == 0: #dan is er niks gevonden
                if sea:
                    x1 = loodrechtXY(inPoint.X,inPoint.Y,inAngle, maxAfstand)[0]
                    y1 = loodrechtXY(inPoint.X,inPoint.Y,inAngle, maxAfstand)[1]
                    coords = [x1,y1]
                elif left == True:
                    x1 = inPoint.X - loodrechtX(inAngle, maxAfstand)
                    y1 = inPoint.Y + loodrechtY(inAngle, maxAfstand)
                    coords = [x1,y1]
                else:
                    x1 = inPoint.X + loodrechtX(inAngle, maxAfstand)
                    y1 = inPoint.Y - loodrechtY(inAngle, maxAfstand)
                    coords = [x1,y1]
            else:
                #dan is er een of meerdere punten gevonden
                #bepaal eerst op het punt niet op de lijn ligt
                for nIndex in nearestIndices: #doe dit voor alle gevonden punten
                    n = nears[nIndex]
                    if isNearestNotSimilar(n, [inPoint.X, inPoint.Y]): #dichtbij, maar niet erop
                        coords = n
                        break
                #er is er geen dichtbij, maar niet erop gevonden, dus dan ook weer op 10m afstand
                if len(coords) == 0:
                    if sea:
                        x1 = loodrechtXY(inPoint.X,inPoint.Y,inAngle, maxAfstand)[0]
                        y1 = loodrechtXY(inPoint.X,inPoint.Y,inAngle, maxAfstand)[1]
                        coords = [x1,y1]       
                    elif left == True:
                        x1 = inPoint.X - loodrechtX(inAngle, maxAfstand)
                        y1 = inPoint.Y + loodrechtY(inAngle, maxAfstand)
                        coords = [x1,y1]
                    else:
                        x1 = inPoint.X + loodrechtX(inAngle, maxAfstand)
                        y1 = inPoint.Y - loodrechtY(inAngle, maxAfstand)
                        coords = [x1,y1]
        else:
            if sea:
                x1 = loodrechtXY(inPoint.X,inPoint.Y,inAngle, maxAfstand)[0]
                y1 = loodrechtXY(inPoint.X,inPoint.Y,inAngle, maxAfstand)[1]
                coords = [x1,y1]         
            elif left == True:
                x1 = inPoint.X - loodrechtX(inAngle, maxAfstand)
                y1 = inPoint.Y + loodrechtY(inAngle, maxAfstand)
                coords = [x1,y1]
            else:
                x1 = inPoint.X + loodrechtX(inAngle, maxAfstand)
                y1 = inPoint.Y - loodrechtY(inAngle, maxAfstand)
                coords = [x1,y1]           
        return coords

def isNearestNotSimilar(a,b, sea=False):   
    if sea:
        sr=arcpy.SpatialReference(4326)
        a = arcpy.PointGeometry(arcpy.Point(x0,y0),sr)
        b = arcpy.PointGeometry(arcpy.Point(x1,y1),sr)
        
        if a.angleAndDistanceTo(b)[1] < 0.001:
            return False
        else:
            return True
    else:
        dist = math.sqrt((b[0] - a[0])**2 + (b[1] - a[1])**2)
        if dist < 0.001:
            return False
        else:
            return True

def defineLine(x0, x1, y0, y1):
    wkt = "LINESTRING ({x0} {y0}, {x1} {y1})".format(x0=x0,
                                                     y0=y0,
                                                     x1=x1,
                                                     y1=y1)
    li = arcpy.FromWKT(wkt)
    return li

def loodRechts(locPoint, angle, distance, sea= False):
    x0 = locPoint.X
    y0 = locPoint.Y
    #Om intersect met de eigen lijn te voorkomen: x0 en x1 5mm verplaatsen
    if sea:
        x1=loodrechtXY(x0,y0,angle,distance)[0]
        x2 = loodrechtXY(x0,y0,angle,0.005)[0]
        y1=loodrechtXY(x0,y0,angle,distance)[1]
        y2=loodrechtXY(x0,y0,angle,0.005)[1]      
    else:
        x1 = x0 + loodrechtX(angle, distance)
        x2 = x0 + loodrechtX(angle, 0.005)
        y1 = y0 - loodrechtY(angle, distance)
        y2 = y0 - loodrechtY(angle, 0.005)
        lineArr = defineLine(x1,x2,y1,y2)
    return lineArr

def loodLinks(locPoint, angle, distance, sea=False):
    x0 = locPoint.X
    y0 = locPoint.Y
    #Om intersect met de eigen lijn te voorkomen: x0 en x1 5mm verplaatsen
    if sea:
        x1=loodrechtXY(x0,y0,angle,distance)[0]
        x2 = loodrechtXY(x0,y0,angle,0.005)[0]
        y1=loodrechtXY(x0,y0,angle,distance)[1]
        y2=loodrechtXY(x0,y0,angle,0.005)[1]      
    else:
        x1 = x0 - loodrechtX(angle, distance)
        x2 = x0 - loodrechtX(angle, 0.005)
        y1 = y0 + loodrechtY(angle, distance)
        y2 = y0 + loodrechtY(angle, 0.005)
        lineArr = defineLine(x1,x2,y1,y2)
    return lineArr

def createLoodlijn(inPoint,inAngle,inDist,left=False,sea=False):
    #1. creer lijn loodrecht op hoogtelijn. Dit is een array
    if sea and left:
        inAngle=(inAngle+180)%360
    if sea:
        inAngle=(inAngle+90)%360
    if left == True:
        lineGeom = loodLinks(inPoint,inAngle,inDist, sea)
    else:
        lineGeom = loodRechts(inPoint,inAngle,inDist,sea)
    return lineGeom

def createLoodLijnTuple(lijn, vertexPoint, loc, unique):
    inRow = [None]*6
    inRow[0] = lijn
    inRow[1] = vertexPoint.Vertex
    inRow[2] = vertexPoint.Kenmerk
    inRow[3] = vertexPoint.Nummer
    inRow[4] = loc
    inRow[5] = unique
    return inRow

def AddMessage(message):
    #print(message)
    arcpy.AddMessage(message)

def AddError(message):
    #print('Error: ' + message)
    arcpy.AddError(message)

def AddWarning(message):
    #print('Warning: ' + message)
    arcpy.AddWarning(message)

def defineAngle(x0, x1,y0,y1, sea=False):  
    if sea:
        x0=math.radians(x0)
        x1=math.radians(x1)
        y0=math.radians(y0)
        y1=math.radians(y1)
        dLon = (x1 - x0)

        x = math.sin(dLon) * math.cos(y1)
        y = math.cos(y0) * math.sin(y1) - math.sin(y0) * math.cos(y1) * math.cos(dLon)

        A = math.atan2(x,y)
        d = math.degrees(A)
        if d < 0:
            d += 360
        return d

    else:
        distX = x1 - x0
        distY = y1 - y0
        #distX = math.cos(x0) * (x1 - x0)
        A = math.atan2(distY,distX)
        d = math.degrees(A)
        if d < 0:
            d += 360
        return d

def findQuadrant(angle):
    if angle < 0:
        angle = angle + 360
    if angle > 360:
        angle = angle - 360
    if angle < 90:
        return 1
    elif angle < 180:
        return 2
    elif angle < 270:
        return 3
    else: return 4
        
def defineHalfAngle(oldAngle,newAngle):
    angle = (oldAngle + newAngle) / 2
    oldQuad = findQuadrant(oldAngle)
    newQuad = findQuadrant(newAngle)
    
    if oldQuad == 1 and newQuad == 3:
        if (oldAngle + 180) < newAngle:
            angle = angle - 180
    if oldQuad == 2 and newQuad == 4:
        if (oldAngle + 180) < newAngle:
            angle = angle - 180
            
    if oldQuad == 3 and newQuad == 1:
        if (oldAngle - 180) > newAngle:
            angle = angle - 180
    if oldQuad == 4 and newQuad == 2:
        if (oldAngle - 180) > newAngle:
            angle = angle - 180
    if oldQuad == 1 and newQuad == 4:
        angle = angle - 180
    if oldQuad == 4 and newQuad == 1:
        angle = angle - 180
    return angle

def defineLength(x0,x1,y0,y1, sea=False):   
    if sea:
        sr=arcpy.SpatialReference(4326)
        a = arcpy.PointGeometry(arcpy.Point(x0,y0),sr)
        b = arcpy.PointGeometry(arcpy.Point(x1,y1),sr)
        
        return a.angleAndDistanceTo(b)[1]

    else:    
        distX =x1 - x0
        distY = y1 - y0
        return math.sqrt((distX**2 + distY**2))

def loodrechtX(ang, dist):
    return dist * math.sin(math.radians(ang))

def loodrechtY(ang, dist):
    return dist * math.cos(math.radians(ang))

def loodrechtXY(x,y,ang,dist):
    sr=arcpy.SpatialReference(4326)
    a = arcpy.PointGeometry(arcpy.Point(x,y),sr)
    point=a.pointFromAngleAndDistance(ang,dist,"GEODESIC")
    x1=point.centroid.X
    y1=point.centroid.Y
    return [x1, y1]
#insert point to cursor
#coord:[x,y], pt = gp point object, r = insertcursor row, c = insertcursor
def insertPoint2Cur(coord,pt,r):
    pt.X = coord[0]
    pt.Y = coord[1]
    r.setValue("Shape", pt)

def insertOtherIntermediateVals(r,un,l,vert,num,km):
    r.setValue(unField,un)
    r.setValue(locField,l)
    r.setValue(vtField,vert)
    r.setValue(nmField,num)
    r.setValue(kmField,km)
    
def createPointFc(inName):
    arcpy.CreateFeatureclass_management(os.path.dirname(inName),os.path.basename(inName),"POINT")
    arcpy.AddField_management(inName,kField,"FLOAT")
    arcpy.AddField_management(inName,rField,"FLOAT")
    arcpy.AddField_management(inName,lField,"FLOAT")
    arcpy.AddField_management(inName,tField,"FLOAT")    
    arcpy.AddField_management(inName,nmField,"FLOAT")
    arcpy.AddField_management(inName,kmField,"TEXT")
    arcpy.AddField_management(inName,vtField,"LONG")

def drawLoodlijnen(pointDict,tinEdgeFc,intersectFc,storeLoodFc="",sea=False):
    loodLines = "in_memory/loodrechtlijnen"
    arcpy.CreateFeatureclass_management("in_memory","loodrechtlijnen","POLYLINE")
    arcpy.AddField_management(loodLines,vtField,"LONG")
    arcpy.AddField_management(loodLines,nmField,"LONG")
    arcpy.AddField_management(loodLines,unField,"LONG")
    arcpy.AddField_management(loodLines,locField,"TEXT")
    arcpy.AddField_management(loodLines,kmField,"TEXT")    
    ic = arcpy.da.InsertCursor(loodLines, ["Shape@", vtField, kmField, nmField, locField, unField])
    
    uniqueNumber = 0
    totProg = 0
    for nummer in pointDict:
        pointList = pointDict[nummer]
        i = 1
        while i < len(pointList):
            totProg+=1
            if (totProg % 50) == 0:
                AddMessage("%s perpendicular lines created..." % str(totProg))
            pd = pointList[i]    
            oldPd = pointList[i-1]        
            if i ==1:
                angle = defineAngle(oldPd.Point.X,pd.Point.X,oldPd.Point.Y,pd.Point.Y, sea=sea)
                right = createLoodlijn(oldPd.Point,angle,maxAfstand)
                loodlijntuple = createLoodLijnTuple(right, oldPd, "r", uniqueNumber)
                ic.insertRow(loodlijntuple)
                left = createLoodlijn(oldPd.Point,angle,maxAfstand,True ,sea=sea)
                loodlijntuple = createLoodLijnTuple(left, oldPd, "l", uniqueNumber)
                ic.insertRow(loodlijntuple)
                uniqueNumber += 1

            #Check of pd de laatste vertex op de lijn is
            elif i == (len(pointList)-1): #ja: loodrecht op het laatste segment
                angle = defineAngle(oldPd.Point.X,pd.Point.X,oldPd.Point.Y,pd.Point.Y , sea=sea)
                length = defineLength(oldPd.Point.X,pd.Point.X,oldPd.Point.Y,pd.Point.Y , sea=sea)
            else: #anders halve hoek tussen de twee segmenten
                newPd = pointList[i+1]
                oldAngle = defineAngle(oldPd.Point.X,pd.Point.X,oldPd.Point.Y,pd.Point.Y ,sea=sea)
                newAngle = defineAngle(pd.Point.X,newPd.Point.X,pd.Point.Y,newPd.Point.Y ,sea=sea)
                angle = defineHalfAngle(oldAngle,newAngle)

            right = createLoodlijn(pd.Point,angle,maxAfstand, sea=sea)
            loodlijntuple = createLoodLijnTuple(right, pd, "r", uniqueNumber)
            ic.insertRow(loodlijntuple)            
            left = createLoodlijn(pd.Point,angle,maxAfstand,True, sea=sea)
            loodlijntuple = createLoodLijnTuple(left, pd, "l", uniqueNumber)
            ic.insertRow(loodlijntuple)               
            uniqueNumber+=1
            i+=1        
    del ic
    
    intersectString = ("%s;%s" %(loodLines,tinEdgeFc))
    arcpy.Intersect_analysis(intersectString, intersectFc, join_attributes="ALL",
                             output_type="POINT")
    if storeLoodFc != "":
        arcpy.CopyFeatures_management(loodLines,storeLoodFc)
    return loodLines

def deriveTinEdges(inTin,output):
    #Aanmaken tin-edges met alleen de hardlines
    surfDesc = arcpy.Describe(inTin)
    tempTinCreated = False
    if surfDesc.datasetType.lower() == "terrain":
        terr = inTin
        inTin = os.path.join(os.path.dirname(output),"tin")
        if arcpy.Exists(inTin):
            arcpy.Delete_management(inTin)
        arcpy.TerrainToTin_3d(terr,inTin, "0", "500000000", "True")
        tempTinCreated = True
    AddMessage("Deriving edges from TIN...")
    arcpy.TinEdge_3d(inTin,output, 'ENFORCED')
    if tempTinCreated == True:
        arcpy.Delete_management(inTin)

def createPointDictionary(lijnFc, pointFc ,sea=False):
    lines = arcpy.da.SearchCursor(lijnFc, field_names=[nmField, kmField, "Shape@"])
    pdDict = {}
    rcnt = 0
    for line in lines:            
        vertex = 1 
        nummer = line[0]
        kenmerk= line[1]
        feat = line[2]
        
        coordArray = []
        pointsArray = []
        pc = arcpy.da.SearchCursor(pointFc,
                                   field_names=["Shape@", kField],
                                   where_clause='{} = {}'.format(nmField, nummer))
        for pr in pc:
            pnt = pr[0].getPart()
            pxy = (pnt.X, pnt.Y)
            coordArray.append(pxy)
            pointData = PointData(pnt, nummer, None, pr[1], kenmerk) # Todo kenmerk weghaken.

            pointsArray.append(pointData)
        del pc
        
        if len(coordArray) == 0:
            AddError("Line with NUMBER = {} has not points. Please fix.".format(nummer))
            return False

        coordNumpyArray = numpy.array(coordArray)

        pdList = []
        
        prt = feat.getPart(0)
        for pnt in prt:
            x = pnt.X
            y = pnt.Y
            coord = (x,y)
            pointIndex = findIndex_nearest_vector(coordNumpyArray, coord)
            pointPoint = pointsArray[pointIndex].Point

            if defineLength(x, pointPoint.X, y, pointPoint.Y, sea=sea) > 0.001: #dan is het een punt dat niet op de vertex ligt
                AddError("Line with NUMBER = %s has (one ore more) point without a vertix." %str(nummer))
                return False
            
            pointElev = pointsArray[pointIndex].Kruin
                        
            pd = PointData(pnt, nummer, vertex, pointElev, kenmerk) # Todo kenmerk weghaken.
            pdList.append(pd)
            vertex += 1
        if (vertex -1) != len(coordArray): #dan zijn het aantal punten en vertices niet gelijk
            AddError("Number of points and verticis with NUMBER = {num} are not equal.".format(num=nummer))
            return False
          
        pdDict[nummer] = pdList
        rcnt += 1
        if (rcnt % 1000) == 0:
            AddMessage("{0} rows processed".format(rcnt))

    del lines
    return pdDict

def findIndex_nearest_vector(arr, value):
    dist = []
    for (diffX, diffY) in (arr-value):
        dist.append(diffX**2 + diffY**2)
    index = numpy.array(dist).argmin()
    del dist
    return index


def PrikTeenhoogte(lijnen,surf,maxDist,crestWidth,ignoreIntersects=False, sea=False):
    global maxAfstand
    maxAfstand = maxDist
    global useStandardDistance
    ds = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
    
    # Find all FID's of bandijken
    bandijkenFids = []
    
    if "elevated_line_" in lijnen.lower():
        cursorSearch = arcpy.da.SearchCursor(lijnen, ['OID@'], where_clause='TYPE = 3')
        for line in cursorSearch:
            bandijkenFids.append(line[0])
    
    bandijkenFids = []
    
    boolBandijk = False

    if "breaklines_" in lijnen.lower(): 
        AddError("Breuklijn Baseline feature class selected. No toe-elevations needed. Process aborted.")
        return

    validFilenames = ["terrain_jump_", "elevated_line_"]
    isValid = False
    for fn in validFilenames:
        if fn in lijnen.lower():
            isValid = True

    if isValid == False:
        AddError("No valid Baseline feature class selected. Process aborted")
        return        

    arcpy.env.overwriteOutput = 1
    
    if arcpy.CheckExtension("3D") == "Available":
        arcpy.CheckOutExtension("3D")
    else:
        AddError("3D Analyst is not available. Process aborted")
        return

    surfaceDesc = arcpy.Describe(surf)
    dsType = surfaceDesc.datasetType.lower()
    if dsType != 'terrain' and dsType != 'tin' and dsType != 'rasterdataset':
        AddError("You have not selected a valid elevation model. Please use a Terrain, TIN or rasterdataset.")
        return
        
    saveIntermediate = True

    #Information and configure output names
    lijnenName = os.path.basename(lijnen)
    lijnenName = lijnenName.replace("_lines","")
    inLineDesc = arcpy.Describe(lijnen)
    inLinePath = inLineDesc.CatalogPath
    inLineShapeFieldName = inLineDesc.ShapeFieldName
    punten = inLinePath.replace("_lines","_points")
    fgdb = os.path.dirname(os.path.dirname(inLinePath))
    outPath = os.path.join(os.path.dirname(fgdb),"data","source","toe_elevation")
    if not os.path.exists(outPath):
        os.mkdir(outPath)
    teenhoogteLocatieShape = os.path.join(outPath,("toe_location_points_%s_%s.shp" %(lijnenName,ds)))
    if saveIntermediate == True:
        loodLocShape = os.path.join(outPath,("perpendicular_lines_%s_%s.shp" %(lijnenName,ds)))
    else:
        loodLocShape = ""
    intersects = os.path.join(outPath,("intersection_points_%s_%s.shp" %(lijnenName,ds)))
    tinEdge = os.path.join(outPath,("tin_edges_%s_%s.shp" %(lijnenName,ds)))
    inmemPoints = "in_memory/teenhoogtes"

    #nodata value
    nodata = -9999

    #set geoprocessing extent
    ext = inLineDesc.Extent
    arcpy.env.extent = arcpy.Extent(ext.XMin - 250,
                                    ext.YMin - 250,
                                    ext.XMax + 250,
                                    ext.YMax + 250)
      
    #Maken lijst met punten obv lijnen en punten
    AddMessage("Find lines and corresponding points..")
    pointDictionary = createPointDictionary(lijnen, punten ,sea=sea)
    if pointDictionary == False:
        AddWarning("Each point sould have a vertex and each vertrex should have a point.")
        return
                 
    #Bepaal type bodemhoogte
    if ignoreIntersects:
        tinEdge = ""
        useStandardDistance = True            
    elif dsType == "terrain" or dsType == "tin":    
        #Afleiden TIN edges
        deriveTinEdges(surf,tinEdge)
        #Maken loodlijnen
        drawLoodlijnen(pointDictionary,tinEdge,intersects,loodLocShape, sea=sea)
    elif dsType == "rasterdataset":
        tinEdge = ""
        useStandardDistance = True
        sea=True
    else:
        AddError("No valid elevation model selected. Please use a Terrain, TIN or rasterdataset.")
        return

    #dictMid wordt gebruikt om de oorspronkelijke
    #teenhoogtes weer te kunnen pakken
    dictMid = {}

    AddMessage("Create temporary points...")
    arcpy.CreateFeatureclass_management(os.path.dirname(inmemPoints),os.path.basename(inmemPoints),"POINT")
    arcpy.AddField_management(inmemPoints, locField, "TEXT")
    arcpy.AddField_management(inmemPoints, kmField, "TEXT") # Todo mag weg
    arcpy.AddField_management(inmemPoints, vtField, "LONG")
    arcpy.AddField_management(inmemPoints, nmField, "LONG")
    arcpy.AddField_management(inmemPoints, unField, "LONG")
    # Dit Z-veld wordt met InterpolateShape overschreven op de locaties die binnen het opgegeven DEM vallen
    # Als een punt buiten de DEM ligt, kreeg deze eerst Z=0, dit is niet makkelijk af te vangen, dus vandaar 
    # nieuw veld maklen en default waarde toekennen
    arcpy.AddField_management(inmemPoints, 'Z', 'DOUBLE')
    nodataZ = -9999
    insectCur = arcpy.da.InsertCursor(inmemPoints, ["Shape@", unField, locField, vtField, nmField, kmField, 'Z']) # todo remove kenmerk

    totProg = 0
    unique = 0

    for nummer in pointDictionary:
        i = 1
        pointList = pointDictionary[nummer]
        while i < len(pointList):
            pd = pointList[i]
            totProg+=1
            if (totProg % 50) == 0:
                AddMessage("%s points processed..." % str(totProg))
        
            oldPd = pointList[i-1]

            if i == 1:
                #plaats punt op eerste vertex
                #midden
                insectRow = [arcpy.Point(oldPd.Point.X, oldPd.Point.Y),
                             unique, "m", oldPd.Vertex, oldPd.Nummer, oldPd.Kenmerk, nodataZ] # todo remove kenmerk
                insectCur.insertRow(insectRow)               
                
                dictMid[unique] = oldPd.Kruin

                firstAngle = defineAngle(oldPd.Point.X,pd.Point.X,oldPd.Point.Y,pd.Point.Y, sea=sea)
                length = defineLength(oldPd.Point.X,pd.Point.X,oldPd.Point.Y,pd.Point.Y, sea=sea)         
                right = getIntersectResults(oldPd.Point,oldPd.Vertex,oldPd.Nummer,firstAngle,maxAfstand,intersects, sea=sea)
                left = getIntersectResults(oldPd.Point,oldPd.Vertex,oldPd.Nummer,firstAngle,maxAfstand,intersects,True, sea=sea)
                if len(right) > 0:
                    insectRow = [arcpy.Point(right[0], right[1]),
                                 unique, "r", oldPd.Vertex, oldPd.Nummer, oldPd.Kenmerk, nodataZ]
                    insectCur.insertRow(insectRow)  
                    
                    # Als het een bandijk is, plaats dan ook een punt op het uiteinde van de lijn op standaard afstand, nodig voor bepalen 
                    #    welke zijde buiten de modelgrens ligt
                    if oldPd.Nummer in bandijkenFids:
                    #if boolBandijk:
                        bright = getIntersectResults(oldPd.Point,oldPd.Vertex,oldPd.Nummer,firstAngle,maxAfstand,intersects, left=False, isBandijk=True, sea=sea)
                        insectRow = [arcpy.Point(bright[0], bright[1]),
                                     unique, "br", oldPd.Vertex, oldPd.Nummer, oldPd.Kenmerk, nodataZ]
                        insectCur.insertRow(insectRow)                       
                #links
                if len(left) > 0:
                    insectRow = [arcpy.Point(left[0], left[1]),
                                 unique, "l", oldPd.Vertex, oldPd.Nummer, oldPd.Kenmerk, nodataZ]
                    insectCur.insertRow(insectRow)
                    if oldPd.Nummer in bandijkenFids:
                    #if boolBandijk:
                        bleft = getIntersectResults(oldPd.Point,oldPd.Vertex,oldPd.Nummer,firstAngle,maxAfstand,intersects,left=True, isBandijk=True)
                        insectRow = [arcpy.Point(bleft[0], bleft[1]),
                                     unique, "bl", oldPd.Vertex, oldPd.Nummer, oldPd.Kenmerk, nodataZ]
                        insectCur.insertRow(insectRow)                      
                unique += 1

            #plaats punt op volgende vertices
            #midden          
            insectRow = [arcpy.Point(pd.Point.X, pd.Point.Y),
                         unique, "m", pd.Vertex, pd.Nummer, pd.Kenmerk, nodataZ]
            insectCur.insertRow(insectRow)   

            dictMid[unique] = pd.Kruin

            #Check of pd de laatste vertex op de lijn is
            if i == (len(pointList)-1): #ja: loodrecht op het laatste segment
                angle = defineAngle(oldPd.Point.X,pd.Point.X,oldPd.Point.Y,pd.Point.Y, sea=sea)
                length = defineLength(oldPd.Point.X,pd.Point.X,oldPd.Point.Y,pd.Point.Y, sea=sea)
            else: #anders halve hoek tussen de twee segmenten
                newPd = pointList[i+1]
                oldAngle = defineAngle(oldPd.Point.X,pd.Point.X,oldPd.Point.Y,pd.Point.Y, sea=sea)
                newAngle = defineAngle(pd.Point.X,newPd.Point.X,pd.Point.Y,newPd.Point.Y, sea=sea)
                angle = defineHalfAngle(oldAngle,newAngle)
                 
            right = getIntersectResults(pd.Point,pd.Vertex,pd.Nummer,angle,maxAfstand,intersects, sea=sea)
            if len(right) > 0:
                insectRow = [arcpy.Point(right[0], right[1]),
                             unique, "r", pd.Vertex, pd.Nummer, pd.Kenmerk, nodataZ]
                insectCur.insertRow(insectRow)  
                
            bright = getIntersectResults(pd.Point,pd.Vertex,pd.Nummer,angle,maxAfstand,intersects, left=False, isBandijk=True)  
            insectRow = [arcpy.Point(bright[0], bright[1]),
                         unique, "br", pd.Vertex, pd.Nummer, pd.Kenmerk, nodataZ]
            insectCur.insertRow(insectRow)              
            #links
            left = getIntersectResults(pd.Point,pd.Vertex,pd.Nummer,angle,maxAfstand,intersects,True, sea=sea)
            bleft = getIntersectResults(pd.Point,pd.Vertex,pd.Nummer,angle,maxAfstand,intersects, left=True, isBandijk=True, sea=sea)
            if len(left) > 0:
                insectRow = [arcpy.Point(left[0], left[1]),
                             unique, "l", pd.Vertex, pd.Nummer, pd.Kenmerk, nodataZ]
                insectCur.insertRow(insectRow)  
            insectRow = [arcpy.Point(bleft[0], bleft[1]),
                         unique, "bl", pd.Vertex, pd.Nummer, pd.Kenmerk, nodataZ]
            insectCur.insertRow(insectRow)                  
            unique += 1
            i+=1

    AddMessage("Get elevation...")   
    arcpy.AddSurfaceInformation_3d(in_surface=surf,
                                   in_feature_class=inmemPoints,
                                   out_property="Z")
        
    AddMessage("Copy teenelevation locations to %s..." % teenhoogteLocatieShape)
    if saveIntermediate == True:
        arcpy.CopyFeatures_management(inmemPoints,teenhoogteLocatieShape)

    dictLinks = {}
    dictRechts = {}

    AddMessage("Filling point file with elevation data...")
    #Lijsten maken met teenhoogtes
    # En een dict maken van alle unieke nummers voor bandijk als deze bestaat
    #   om later te kunnen bepalen welke kant van de bandijk buiten de modelgrens ligt
    dictBandijk = {}
    pointCur = arcpy.da.SearchCursor(inmemPoints,
                                     field_names=[locField, unField, "Z", nmField], 
                                     where_clause=("\"%s\" = 'l' OR \"%s\" = 'r' OR \"%s\" = 'bl' OR \"%s\" = 'br'" %(locField,locField, locField, locField)))
    for point in pointCur:
        loc = point[0]
        id = point[1]
        val = point[2]
        nm = point[3]
        if loc == "l":
            dictLinks[id] = val
        elif loc == "r":
            dictRechts[id] = val
        elif loc in ['bl', 'br'] and nm in bandijkenFids:
        #elif loc in ['bl', 'br'] and boolBandijk:
            if not nm in dictBandijk:
                dictBandijk[nm] = {'bl': 0, 'br': 0, 'teen': 'links'}
            if point[2] < nodataZ:
                dictBandijk[nm][loc] += 1                           
    del pointCur
    
    # Dict bandijk aanvullen met welke zijde de teen ligt (dus welke zijde aan modelzijde ligt)
    for nm in dictBandijk:
        bdnm = dictBandijk[nm]
        if bdnm['bl'] > bdnm['br']:
            bdnm['teen'] = 'links'
        elif bdnm['br'] > bdnm['bl']:
            bdnm['teen'] = 'rechts'            
        else:
            bdnm['teen'] = 'links'        
    

    #Hier een layer maken, waarna een selectie wordt gemaakt met alle nummers in puntdict
        #die wegknikkeren
    AddMessage("Prepare by deleting old data...")

    rcnt = 0
    # Was: via een select layer, maar dit duurde verschrikkelijk lang
    # nu: een updatecursor die rijen weggooit
    dc = arcpy.da.UpdateCursor(punten, [nmField])
    for row in dc:
        if row[0] in pointDictionary:
            dc.deleteRow()
        rcnt += 1
        if (rcnt % 10000) == 0:
            AddMessage("{0} rows processed".format(rcnt))

    dictDrempelLinks = {}
    dictDrempelRechts = {}

    """
    if boolIncludeTeen == True:
        if boolBandijk == False:
            insertFields = ["Shape@", nmField, kmField, kField, lField, rField]
        else:
            insertFields = ["Shape@", nmField, kmField, kField, tField]
        insertFields = ["Shape@", nmField, kmField, kField, lField, rField]
    else:
        insertFields = ["Shape@", nmField, kmField, kField, tField]
    """
    # insertFields = ["Shape@", nmField, kmField, kField, lField, rField]
    insertFields = ["Shape@", nmField, kField, lField, rField]

    if len(arcpy.ListFields(punten, kbField)):
        insertFields.append(kbField)
        haskb = True
    else:
        haskb = False

    if len(arcpy.ListFields(punten, rhField)):
        insertFields.append(rhField)
        hasrh = True
    else:
        hasrh = False

    if len(arcpy.ListFields(punten, tlField)):
        insertFields.append(tlField)
        hastl = True
        
    else:
        hastl = False

    if len(arcpy.ListFields(punten, trField)):
        insertFields.append(trField)
        hastr = True
    else:
        hastr = False

    insertCur = arcpy.da.InsertCursor(punten, field_names=insertFields)            
    pointCur = arcpy.da.SearchCursor(inmemPoints,
                               field_names=["Shape@", nmField, unField, "Z"], 
                               where_clause=("\"%s\" = 'm'"%(locField)))
    for point in pointCur:
        newRow = [None]*len(insertFields)
        newRow[findfieldindex(insertCur, "Shape@")] = point[0]
        kh = None
        nm = point[1]
        newRow[findfieldindex(insertCur, nmField)] = nm
        un = point[2]
        if not dictMid[un] is None:
            kh = dictMid[un]
            newRow[findfieldindex(insertCur, kField)] = kh
        else:
            kh = point[3]
            newRow[findfieldindex(insertCur, kField)] = kh

        #newRow[findfieldindex(insertCur, kmField)] = point[4]
        
        if nm not in bandijkenFids:
            lh = dictLinks[un]
            rh = dictRechts[un]
            
            if lh < -100000 or lh > kh:
                lh = kh            
            newRow[findfieldindex(insertCur, lField)] = lh                
            #Bepaling linkerhoogte
            if not dictDrempelLinks.has_key(nm):
                dictDrempelLinks[nm] = [(kh - lh)]
            else:
                dictDrempelLinks[nm].append(kh - lh)
            #Bepaling rechterhoogte
            if rh < -100000 or rh > kh:
                rh = kh
            newRow[findfieldindex(insertCur, rField)] = rh
            
            if not dictDrempelRechts.has_key(nm):
                dictDrempelRechts[nm] = [(kh - rh)]        
            else:
                dictDrempelRechts[nm].append(kh - rh)

        else:
            if dictBandijk[nm]['teen'] == 'links':
                #th = dictLinks[un]
                newRow[findfieldindex(insertCur, lField)] = dictLinks[un]
                newRow[findfieldindex(insertCur, rField)] = kh
            else:
                #th = dictRechts[un]
                newRow[findfieldindex(insertCur, rField)] = dictRechts[un]
                newRow[findfieldindex(insertCur, lField)] = kh
            
        if hastl == True:
            newRow[findfieldindex(insertCur, tlField)] = defaultTalud

        if hastr == True:
            newRow[findfieldindex(insertCur, trField)] = defaultTalud

        if hasrh == True:
            newRow[findfieldindex(insertCur, rhField)] = defaultRuwheid  

        if haskb == True:
            newRow[findfieldindex(insertCur, kbField)] = crestWidth       
      
        insertCur.insertRow(newRow)
    del pointCur, insertCur

    if lijnen.count("terrain_jump_") > 0:
        AddMessage("Setting sill at correct side of hoogteverschillijn...") 
        for nm in dictDrempelLinks:
            l = dictDrempelLinks[nm]
            r = dictDrempelRechts[nm]
            dl = (sum(l) / len(l))
            dr = (sum(r) / len(r))       
            
            if dr > dl: #dan moet links gelijk zijn aan kruin
                setTeenSideIsKruin(punten, lField, nm)
            else:#dan moet rechts gelijk zijn aan kruin
                setTeenSideIsKruin(punten, rField, nm)

##############################################################################################
##
##############################################################################################    

if __name__ == "__main__":
    surf = arcpy.GetParameterAsText(1)
    lijnen = arcpy.GetParameterAsText(0)
    maxAfstand = arcpy.GetParameter(2)
    ignoreIntersects = arcpy.GetParameter(3)
    
    #lijnen = 'C:\\baseline\\spelen\\testen\\testset_b5\\myBL6collection\\measures_baseline6\\maatregelen\\nr_doorwcl_a2\\baseline.gdb\\elevation\\terrainjumps_lines'
    #surf = 'C:\\baseline\\spelen\\testen\\testset_b5\\myBL6collection\\ref\\baseline.gdb\\elevation\\elevation_model'
    #ignoreIntersects = False
    #maxAfstand = 10
    if "elevated" in lijnen:
       crestWidth = 3
    else:
       crestWidth = 10

    PrikTeenhoogte(lijnen,surf,maxAfstand, crestWidth,ignoreIntersects)
