import os
import arcpy
import settings
from generalFunctions import *
from os.path import join

"""
 
Baseline 6 Input Module

AIM
    - Update characteristics in all files

Lievense 2017
Niels Mulder

Based on bim5_UpdateAllKenmerk.py by Koen Volleberg, Lievense

"""

arcpy.env.overwriteOutput = True

"""
This function transfers check is a FC has a certain field. Case-insensitive

Procedure
    -

Input
    string featureclass = path to FC
    string fieldToCheck = field name.

"""
def featureclassHasField(featureclass, fieldToCheck):
    fieldToCheck = fieldToCheck.lower()
    fields = arcpy.ListFields(featureclass)
    for field in fields:
        if field.name.lower() == fieldToCheck:
            del fields
            return True
    del fields
    return False   

def UpdateCharacteristics(baselineDir, characteristics, measure):

    # string baselineDir = path of baseline dir
    # string characteristics = new value for characteristics-field
    # string measure = new value for measure-fiels

    AddMessage('Baseline Input Module Tool: {}. Version {}'.format(os.path.basename(sys.argv[0]), settings.getVersion()))
    if not settings.isDef():
        AddMessage('This is a beta version. Do not use in production.')
    AddMessage('WSP {}'.format(settings.getVersionDate()))

    arcpy.env.workspace = os.path.join(baselineDir, 'baseline.gdb')

    # List datasets
    datasets = arcpy.ListDatasets()

    # Add None to datasets. Those points to the files not in a dataset
    datasets = [''] + datasets if datasets is not None else []

    # Loop over datasets
    for dataset in datasets:
        for featureClass in arcpy.ListFeatureClasses(feature_dataset=dataset):
            
            path = os.path.join(arcpy.env.workspace, dataset, featureClass)
            
            # Check if FC has characteristics field
            if featureclassHasField(path, "characteristics"):
                # Set field
                arcpy.CalculateField_management(path, "characteristics", '"{}"'.format(characteristics))

            else :
                pass
                #AddWarning("Field characteristics does not exist within {}.".format(featureClass))
    
    if arcpy.Exists("bridge_events"):
        arcpy.CalculateField_management("bridge_events", "characteristics", '"{}"'.format(characteristics))

    # Set MEASURE in measure_contour_polygons
    path = os.path.join(arcpy.env.workspace, 'metadata', 'measure_contour_polygons')
    if arcpy.Exists(path):
        arcpy.CalculateField_management(path, "MEASURE", '"{}"'.format(measure))


if __name__ == "__main__":
    # string baselineDir = path of baseline dir
    # string characteristics = new value for characteristics-field
    # string measure = new value for measure-fiels

    baselineDir = arcpy.GetParameterAsText(0)
    characteristic = arcpy.GetParameterAsText(1)
    measure = arcpy.GetParameterAsText(2)
    UpdateCharacteristics(baselineDir, characteristic, measure)
